package de.inso_gmbh.ipkexportplugin;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * The main plugin class to be used in the desktop.
 */
public class IpkExportPlugin extends AbstractUIPlugin 
{
	//The shared instance.
	private static IpkExportPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	/**
	 * The constructor.
	 */
	public IpkExportPlugin() 
	{
		super();
		plugin = this;
		try 
		{
			resourceBundle= ResourceBundle.getBundle("de.inso_gmbh.ipkgplugin.IpkgpluginPluginResources");
		} 
		catch (MissingResourceException x) 
		{
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static IpkExportPlugin getDefault() 
	{
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() 
	{
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) 
	{
		ResourceBundle bundle= IpkExportPlugin.getDefault().getResourceBundle();
		try 
		{
			return bundle.getString(key);
		} 
		catch (MissingResourceException e) 
		{
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() 
	{
		return resourceBundle;
	}
	
	/**
	 * Creates a image descriptor for a image file. 
	 * 
	 * @param sFileName Name/path of the image file, relatively to the install directory of the plugin.
	 * 
	 * @return Descriptor for the image file.
	 */
	public static ImageDescriptor createImageDescriptor(String sFileName)
	{
		return imageDescriptorFromPlugin("de.insogmbh.ipkexportplugin", sFileName);
	}
}
