/*
 * IPK export plugin for Eclipse. 
 * Copyright (C) 2005  Bernd Lachner, InSo GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * You can reach the authr via mail:
 * 
 * InSo GmbH, Individuelle Software-Erstellung
 * Hannongstrae 5
 * 67227 Frankenthal, Germany
 * 
 * or electronic mail:
 * 
 * info@inso-gmbh.de
 */

package de.inso_gmbh.ipkexportplugin.wizards;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;

import de.inso_gmbh.ipkexportplugin.IpkExportPlugin;
import de.inso_gmbh.swt.widgets.*;

/**
 * The "desktop settings" wizard page allows setting the parameters of the ipk desktop file.
 * 
 * @author Bernd Lachner, InSo GmbH
 *
 */
public class IpkExportWizardPageDesktopSettings extends ExtWizardPage 
{
	private Text m_oApplicationNameText = null; 
	private Text m_oExecNameText = null; 
	private Text m_oCommentText = null; 
	private Text m_oMimeTypeText = null; 
	private Text m_oIconNameText = null; 

	private Text m_oJvmOptionsText	= null;
	private Text m_oMainArgumentsNum = null;

	private ImageViewer m_oIconImageViewer = null;
			
	private Combo m_oTypeCombo = null;

	private static final String m_asType[] = {"Application", "Game", "Jeode", "Setting"};	
	private static final String m_asTypeDir[] = {"Applications", "Games", "Jeode", "Settings"};
	private static final String m_asTypeCombo[] = {"Application", "Game", "Jeode-Application", "Setting-Application"};

	private static final int m_iMaxApplicationNameLen = 50;
		
	/**
	 * Constructor for "desktop settings" wizard page.
	 * 
	 * @param oParent Reference to the parent wizard object.
	 */
	public IpkExportWizardPageDesktopSettings(IpkExportWizard oParent) 
	{
		super("DesktopSettingsPage", "IPK Export / Desktop settings and start script", IpkExportPlugin.createImageDescriptor("icons/ipk_export_wizard.gif"), oParent);
		setDescription("The settings of the desktop file and the start script for the IPK package.");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) 
	{
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Desktop settings:");
		createDesktopSettingsGroup(container);

		label = new Label(container, SWT.NULL);
		label.setText("Start script settings:");
		createStartScriptSettingsGroup(container);
		
		readSettingValuesFromXML(this);
		
		dialogChanged();
		setControl(container);
	}
	
	/**
	 * Creates the widgets for the desktop file parameters.
	 * 
	 * @param oParent Parent widget.
	 */
	protected void createDesktopSettingsGroup(Composite oParent)
	{
		Composite oContainer = makeGridContainer(oParent, 3);
		
		this.m_oApplicationNameText = addLabelAndText(oContainer, "Aplication name:", 2, false);
		this.m_oApplicationNameText.addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent oVerifyEvent)
			{
				verifyFileName(oVerifyEvent);
			}
		});
		this.m_oExecNameText = addLabelAndText(oContainer, "Exec name:", 2, true);
		this.m_oCommentText = addLabelAndText(oContainer, "Comment:", 2, false);
		this.m_oMimeTypeText = addLabelAndText(oContainer, "Mime type:", 2, false);
		this.m_oIconNameText = addLabelAndText(oContainer, "Icon:", 1, false);
		this.m_oIconNameText.addKeyListener(new KeyAdapter() 
		{
			public void keyReleased(KeyEvent oKeyEvent) 
			{
				String sIconName = ((Text)oKeyEvent.widget).getText().trim();
				if (sIconName.toLowerCase().endsWith(".png") == true)
				{
					setSourceIconFileName(sIconName, false);	
				}
				else
				{
					m_oIconImageViewer.clearImage();
				}
			}
		});

		Button oButton = new Button(oContainer, SWT.PUSH);
		oButton.setText("Browse...");
		oButton.addSelectionListener(new SelectionAdapter() 
		{
			public void widgetSelected(SelectionEvent e) 
			{
				handleIconBrowse();
			}
		});
		Label oLabel = new Label(oContainer, SWT.NULL);
		oLabel.setText("Icon preview:");
		GridData oGridData = new GridData(GridData.FILL_BOTH);
		oLabel.setLayoutData(oGridData);
		this.m_oIconImageViewer = new ImageViewer(oContainer, SWT.BORDER);
		oGridData = new GridData();
		oGridData.horizontalSpan = 2;
		oGridData.widthHint = 32;
		oGridData.heightHint = 32;
		this.m_oIconImageViewer.setLayoutData(oGridData);
		
		this.m_oTypeCombo = addLabelAndCombo(oContainer, "Type:", IpkExportWizardPageDesktopSettings.m_asTypeCombo);
	}

	/**
	 * Creates the widgets for the start script settings.
	 * 
	 * @param oParent Parent widget.
	 */
	protected void createStartScriptSettingsGroup(Composite oParent)
	{
		Composite oContainer = makeGridContainer(oParent, 2);
		
		this.m_oJvmOptionsText = addLabelAndText(oContainer, "Jvm options:", 1,  false);
		this.m_oMainArgumentsNum = addLabelAndText(oContainer, "Number of main arguments:", 1, false);
		this.m_oMainArgumentsNum.addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent oVerifyEvent)
			{
				verifyMainArgumentsNum(oVerifyEvent);
			}
		});
	}

	/**
	 * Gets the application name parameter for the desktop file.
	 * 
	 * @return Application name.
	 */
	public String getApplicationName()
	{
		return this.m_oApplicationNameText.getText();
	}	

	/**
	 * Sets the application name parameter for the desktop file.
	 * 
	 * @param sApplicationName Name of the application.
	 */
	public void setApplicationName(String sApplicationName)
	{
		this.m_oApplicationNameText.setText(sApplicationName);
	}	

	/**
	 * Gets the executable name parameter for the desktop file.
	 * This is also the name of the start script.
	 * 
	 * @return Executable name.
	 */	
	public String getExecName()
	{
		return this.m_oExecNameText.getText();
	}
	
	/**
	 * Gets the name for the jar file. This name is build as follows:
	 * 
	 * executable name.jar
	 * 
	 * @return Name of the jar file.
	 */
	public String getJarName()
	{
		return this.m_oExecNameText.getText() + ".jar";
	}
	
	/**
	 * Gets the name of the desktop file. This name is build as follows:
	 * 
	 * executable name.desktop
	 * 
	 * @return Name of the desktop file.
	 */
	public String getDesktopFileName()
	{
		return this.m_oExecNameText.getText() + ".desktop";
	}

	/**
	 * If a icon is set this method returns the Icon parameter for the desktop file, otherwise
	 * a empty string is returned. This name is build as follows:
	 * 
	 * executable name.png
	 * 
	 * @return Icon name.
	 */
	public String getIconFileName()
	{
		if (this.m_oIconImageViewer.getImage() != null && isTextEmpty(this.m_oExecNameText) == false)
		{
			return this.m_oExecNameText.getText() + ".png";
		}
		else
		{
			return "";
		}
	}
	
	/**
	 * Gets the comment parameter for the desktop file.
	 * 
	 * @return Comment.
	 */
	public String getComment()
	{
		return this.m_oCommentText.getText();
	}
	
	/**
	 * Sets the comment parameter for the desktop file.
	 * 
	 * @param sComment comment
	 */
	public void setComment(String sComment)
	{
		this.m_oCommentText.setText(sComment);
	}
	
	/**
	 * Gets the mime type parameter for the desktop file.
	 * 
	 * @return Mime type.
	 */
	public String getMimeType()
	{
		return this.m_oMimeTypeText.getText();
	}
	
	/**
	 * Sets the mime type parameter for the desktop file.
	 * 
	 * @param sMimeType mime type
	 */
	public void setMimeType(String sMimeType)
	{
		this.m_oMimeTypeText.setText(sMimeType);
	}
		
	/**
	 * Gets the application type parameter for the desktop file.
	 * 
	 * @return Type.
	 */
	public String getType()
	{
		return getComboValue(this.m_oTypeCombo, IpkExportWizardPageDesktopSettings.m_asType);
	}

	/**
	 * Sets the application type parameter for the desktop file.
	 * 
	 * @param sType application type
	 */
	public void setType(String sType)
	{
		setComboValue(m_oTypeCombo, sType, IpkExportWizardPageDesktopSettings.m_asType);
	}
	
	/**
	 * Gets the destination directory for the desktop file dependent of the applications type.
	 * 
	 * @return Desktop file destination directory.
	 */
	public String getTypeDir()
	{
		return getComboValue(this.m_oTypeCombo, IpkExportWizardPageDesktopSettings.m_asTypeDir);
	}	

	/**
	 * Gets the file name including the path of the png icon for the ipk package.
	 * If no icon is set, a empty string is returned.
	 * 
	 * @return Icon file name.
	 */
	public String getSourceIconFileName()
	{
		if (this.m_oIconImageViewer.getImage() != null && isTextEmpty(this.m_oIconNameText) == false)
		{
			return this.m_oIconNameText.getText().trim();
		}
		else
		{
			return "";
		}
	}
	
	/**
	 * Sets the file name including the path of the png icon for the ipk package.
	 * 
	 * @param sIconFileName file name including the path of the png icon
	 */
	public void setSourceIconFileName(String sIconFileName)
	{
		setSourceIconFileName(sIconFileName, true);
	}
	
	/**
	 * Sets the icon file name and displays a preview if possible.
	 * 
	 * @param sIconFileName The icon file name including the path.
	 * @param bSetText If true the text of the widget which shows the icon file name is set or cleared. 
	 * If false the text of the widget will not set or cleared.
	 */
	private void setSourceIconFileName(String sIconFileName, boolean bSetText)
	{
		if (sIconFileName.toLowerCase().endsWith(".png") == true)
		{
			// File with png extension choosen
			this.m_oIconImageViewer.loadImage(sIconFileName);
		}
		else
		{
			// File with wrong extension choosen
			this.m_oIconImageViewer.clearImage();
		}
		if (bSetText)
		{
			if (this.m_oIconImageViewer.getImage() != null)
			{
				this.m_oIconNameText.setText(sIconFileName);
			}
			else	
			{
				this.m_oIconNameText.setText("");
			}
		}
	}
	
	/**
	 * Gets the number of arguments which the start script should give to the programs main method.
	 * 
	 * @return Number of arguments.
	 */
	public int getMainArgumentsNum()
	{
		try
		{
			return Integer.parseInt(this.m_oMainArgumentsNum.getText());
		}
		catch (NumberFormatException oException)
		{
			return 0;
		}

	}	

	public void setMainArgumentsNum(int iMainArgumentsNum)
	{
		m_oMainArgumentsNum.setText(Integer.toString(iMainArgumentsNum));
	}
	
	/**
	 * Gets the (optional) jvm options.
	 * 
	 * @return Options for the jvm.
	 */
	public String getJvmOptions()
	{
		return this.m_oJvmOptionsText.getText();
	}
	
	/**
	 * Sets the (optional) jvm options.
	 * 
	 * @param sJVMOptions JVM options
	 */
	public void setJvmOptions(String sJVMOptions)
	{
		this.m_oJvmOptionsText.setText(sJVMOptions);
	}
	
	/**
	 * A verify handle method for a text widget. It verifies if the text of the widget only 
	 * contains positive numbers.
	 * 
	 * @param oVerifyEvent Verify event.
	 */
	protected void verifyMainArgumentsNum(VerifyEvent oVerifyEvent) 
	{
		try
		{
			if (oVerifyEvent.text.length() == 0)
			{
				oVerifyEvent.doit = true;
				return;
			}
			int iMainArgumentsNum = Integer.parseInt(oVerifyEvent.text);
			if (iMainArgumentsNum < 0)
				oVerifyEvent.doit = false;
			else
				oVerifyEvent.doit = true;
		}
		catch (NumberFormatException oException)
		{
			oVerifyEvent.doit = false;
		}

	}
	
	/**
	 * Is called when the dialog settings have changed.
	 */
	protected void dialogChanged() 
	{
		this.m_oExecNameText.setText(this.getApplicationName().toLowerCase());
		if (isTextEmpty(this.m_oApplicationNameText))
		{
			updateStatus("Application name is required!");
			return;
		}
		if (getApplicationName().length() > m_iMaxApplicationNameLen)
		{
			updateStatus("Application name is too long!");
			return;		
		}
		updateStatus(null);
	}

	/**
	 * Handle methode for the icon browse button.
	 */
	protected void handleIconBrowse()
	{
		FileDialog oOpenFileDialog = new FileDialog(getShell(), SWT.OPEN);
		String asExtensions[] = new String[1];
		asExtensions[0] = "*.png";
		oOpenFileDialog.setFilterExtensions(asExtensions);
		String sOpenFileName = oOpenFileDialog.open();
		if (sOpenFileName != null)
		{
			setSourceIconFileName(sOpenFileName, true);
		}
	}
	
	/**
	 * @see ExtWizardPage#writeToProjectResources() 
	 */
	protected void writeToProjectResources()
	{
		writeSettingValuesToXML(this);
	}
}