/*
 * IPK export plugin for Eclipse. 
 * Copyright (C) 2005  Bernd Lachner, InSo GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * You can reach the authr via mail:
 * 
 * InSo GmbH, Individuelle Software-Erstellung
 * Hannongstrae 5
 * 67227 Frankenthal, Germany
 * 
 * or electronic mail:
 * 
 * info@inso-gmbh.de
 */

package de.inso_gmbh.ipkexportplugin.wizards;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.SearchRequestor;
import org.eclipse.jdt.ui.JavaElementLabelProvider;
import org.eclipse.jdt.ui.JavaElementSorter;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jdt.ui.StandardJavaElementContentProvider;
import org.eclipse.jdt.ui.jarpackager.JarPackageData;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.inso_gmbh.ipkexportplugin.IpkExportPlugin;

/**
 * The "jar settings" wizard page allows setting the jar parameters for
 * the jar file included in the ipk package.
 * 
 * @author Bernd Lachner, InSo GmbH
 *
 */
public class IpkExportWizardPageJarSettings extends ExtWizardPage implements ICheckStateListener, ISelectionChangedListener
{
	// Widgets
	private Text m_oMainClassText = null; 
	private Button m_oMainClassBrowseButton = null;
	private CheckboxTreeViewer m_oPackageViewer = null;
	protected CheckboxTableViewer m_oResourceViewer = null;
		
	// Object for jar generation	
	private JarPackageData m_oJarPackageData = new JarPackageData();
	
	// Map with selected resources key = selected object in package viewer, value = list of selected objects in ressource viewer
	private HashMap m_oSelectedTableElements = new HashMap();
	// Actual selected object in package viewer
	private Object m_oSelectedPackageElement = null;
	
	/**
	 * Constructor for "jar settings" wizard page.
	 * 
	 * @param oWorkbenchSelection Workbench selection from eclipse.
	 * @param oParent Reference to the parent wizard object.
	 */	
	public IpkExportWizardPageJarSettings(IStructuredSelection oWorkbenchSelection, IpkExportWizard oParent) 
	{
		super("JarSettingsPage", "IPK Export / Jar settings", IpkExportPlugin.createImageDescriptor("icons/ipk_export_wizard.gif"), oParent);
		setDescription("The settings for the Jar file which will be included in the IPK package.");
	}

	/**
	 * Is called when a Item in the package or the resource viewer is checked or unchecked.
	 * 
	 * @param oEvent Check state changed event.
	 */
	public void checkStateChanged(CheckStateChangedEvent oEvent) 
	{
		if (oEvent.getCheckable().equals(this.m_oResourceViewer))
		{
			// Resource viewer (table) element checked or unchecked 
			checkResourceElement(oEvent.getElement(), oEvent.getChecked());
		}
		else
		{
			// Package viewer (tree) element checked or unchecked
			checkPackageElement(oEvent.getElement(), oEvent.getChecked());
		}
		setPackageState();
		dialogChanged();
	}
	
	/**
	 * Add a item which is checked in the resource viewer to the list of selected items.
	 * 
	 * @param oResourceElement The checked item.
	 * @param bChecked True if checked, False if unchecked.
	 */
	private void checkResourceElement(Object oResourceElement, boolean bChecked)
	{
		// Get list of selected resources of the package the element which should be checked or unchecked belongs
		// or create a new list if it don't exist.
		ArrayList listSelected = (ArrayList) this.m_oSelectedTableElements.get(this.m_oSelectedPackageElement);
		if (listSelected == null)
		{
			listSelected = new ArrayList();
		}
		// Add / remove the element wich should be checked / unchecked from the list
		if (bChecked == true)
		{
			if (listSelected.contains(oResourceElement) == false)
			{
				listSelected.add(oResourceElement);
			}
		}
		else
		{
			if (listSelected.contains(oResourceElement) == true)
			{
				listSelected.remove(oResourceElement);
			}
		}
		// If already a item is in the list add the list to map of selected items, otherwise remove the list from the map
		if (listSelected.size() > 0)
		{	
			this.m_oSelectedTableElements.put(this.m_oSelectedPackageElement, listSelected);
		}
		else
		{
			this.m_oSelectedTableElements.remove(this.m_oSelectedPackageElement);
		}
	}
	
	/**
	 * Add a item which is checked in the package viewer and all its childrens to the list of selected items.
	 * 
	 * @param oPackageElement The checked item.
	 * @param bChecked True if checked, False if unchecked.
	 */
	private void checkPackageElement(Object oPackageElement, boolean bChecked)
	{
		// If the package content is viewed in the resource viewer check all items in the resource viewer
		if (this.m_oSelectedPackageElement == oPackageElement)
			this.m_oResourceViewer.setAllChecked(bChecked);
		// Check or uncheck all the content of the package (resources)
		if (bChecked == true)
		{
			ArrayList listSelected = getResourceElements(oPackageElement);
			this.m_oSelectedTableElements.put(oPackageElement, listSelected);			
		}
		else
		{
			this.m_oSelectedTableElements.remove(oPackageElement);
		}
		// Recursivly check or uncheck all childrens
		ArrayList listChildrens = getPackageChildren(oPackageElement);
		Iterator it = listChildrens.iterator();
		while (it.hasNext())
		{
			checkPackageElement(it.next(), bChecked);
		}
	}
	
	/**
	 * Set the check state of the actually selected items in the package viewer.
	 * 
	 * Package which content is only partially selected will be grayed checked,
	 * packages which content is completely selected will be white checked and all
	 * other packages will be unchecked..
	 */
	private void setPackageState()
	{
		// List of all checked package (tree) elments	
		ArrayList listCheckedPackageElements = new ArrayList();
		// Add all selected packages and its parent packages to the list of checked elements
		Iterator it = m_oSelectedTableElements.keySet().iterator();
		while (it.hasNext())
		{
			Object oPackageElement = it.next();
			listCheckedPackageElements.add(oPackageElement);
			while ((oPackageElement = ((ITreeContentProvider)this.m_oPackageViewer.getContentProvider()).getParent(oPackageElement)) != null)
			{
				listCheckedPackageElements.add(oPackageElement);
			}			
		}
		// List of all grayed package (tree) elements
		ArrayList listGrayedPackageElements = new ArrayList();
		// Add all checked package elements which resources are only partially checked to the list of grayed elements
		it = listCheckedPackageElements.iterator();
		while (it.hasNext())
		{
			Object oPackageElement = it.next();
			int iSelectedElements = 0;
			ArrayList listSelected = (ArrayList)this.m_oSelectedTableElements.get(oPackageElement);
			if (listSelected != null)
			{
				iSelectedElements = listSelected.size();
			}			
			if (this.getResourceElements(oPackageElement).size() != iSelectedElements)
			{
				listGrayedPackageElements.add(oPackageElement);
			}
		}
		// Add all checked package elements which childs are only partially checked to the list of grayed elements
		it = listCheckedPackageElements.iterator();
		while (it.hasNext())
		{
			Object oPackageElement = it.next();
			if (listGrayedPackageElements.contains(oPackageElement) == true)
			{
				continue;
			}
			ArrayList listChilds = getPackageChildren(oPackageElement);
			Iterator jt	= listChilds.iterator();
			while (jt.hasNext())
			{
				Object oChild = jt.next();
				if (listCheckedPackageElements.contains(oChild) == false || listGrayedPackageElements.contains(oChild) == true)
				{
					listGrayedPackageElements.add(oPackageElement);
					break;
				}
			}
		}

		this.m_oPackageViewer.setCheckedElements(listCheckedPackageElements.toArray());		
		this.m_oPackageViewer.setGrayedElements(listGrayedPackageElements.toArray());
	}
	
	/**
	 * 
	 * @author Bernd Lachner, InSo GmbH
	 *
	 * Filter class to filter packages for the pacakge viewer.
	 */
	private class PackageViewerFilter extends ViewerFilter
	{
		public boolean select(Viewer oViewer, Object oParentElement, Object oElement) 
		{
			if (oElement instanceof IJavaElement)
			{
				// Filter out non Java Project, Model or Package elements
				IJavaElement JavaElement = (IJavaElement) oElement;
				int iType = JavaElement.getElementType();
				if (iType != IJavaElement.JAVA_MODEL && iType != IJavaElement.JAVA_PROJECT &&
					iType != IJavaElement.PACKAGE_FRAGMENT_ROOT && iType != IJavaElement.PACKAGE_FRAGMENT)
				{
					return false;
				}
				// Filter out empty packages
				if (oElement instanceof IPackageFragment)
				{
					IPackageFragment PackageFragment = (IPackageFragment) oElement;
					try
					{
						return PackageFragment.hasChildren() || (PackageFragment.getNonJavaResources().length > 0);
					}
					catch (JavaModelException e)
					{
						return false;
					}					
				}
				// Filter out Jar files
				if (oElement instanceof IPackageFragmentRoot)
				{
					IPackageFragmentRoot PackageFragmentRoot = (IPackageFragmentRoot) oElement;	
					if (PackageFragmentRoot.isArchive())
						return false;					
				}
				return true;
			}
			return false;
		}
	}
	
	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) 
	{
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Select the resources to export:");
		this.createPackageSelectionGroup(container);
		label = new Label(container, SWT.NULL);
		label.setText("Select application entry point:");
		this.createMainClassSelectionGroup(container);		

		readJarResources();
		readSettingValuesFromXML(this);
		
		dialogChanged();
		setControl(container);
	}

	private void readJarResources()
	{
		NodeList listResourceNodes = m_oParent.getDocumentIPKSettings().getElementsByTagName("Resource");
		// Get content Provider of the Package and the Resource viewer
		ITreeContentProvider iPackageContentProvider = (ITreeContentProvider) m_oPackageViewer.getContentProvider();
		IStructuredContentProvider iResourceContentProvider = (IStructuredContentProvider) m_oResourceViewer.getContentProvider();
		// Get the top level objects shown in Package viewer
		Object aoPackageObjects[] = iPackageContentProvider.getChildren(m_oPackageViewer.getInput());
		checkEnabled(aoPackageObjects, iPackageContentProvider, iResourceContentProvider, listResourceNodes);
		setPackageState();		
	}
	
	private void checkEnabled(Object aoPackageObjects[], ITreeContentProvider iPackageContentProvider, IStructuredContentProvider iResourceContentProvider, NodeList listResourceNodes)
	{
		for (int i = 0; i < aoPackageObjects.length; i++)
		{
			m_oResourceViewer.setInput(aoPackageObjects[i]);
			m_oSelectedPackageElement = aoPackageObjects[i];
			Object aoResourceObjects[] = iResourceContentProvider.getElements(aoPackageObjects[i]);
			for (int j = 0; j < aoResourceObjects.length; j++)
			{
				IResource iResource = null;
				if (aoResourceObjects[j] instanceof IResource)
				{
					iResource = (IResource) aoResourceObjects[j];
				}
				else if (aoResourceObjects[j] instanceof IJavaElement)
				{
					IJavaElement iJavaElement = (IJavaElement) aoResourceObjects[j];
					iResource = iJavaElement.getResource();
				}
				String sResource = iResource.getFullPath().toPortableString();
				if (iResource != null)
				{
					for (int k = 0; k < listResourceNodes.getLength(); k++)
					{
						// if DOM Level 3 / Java 2 SE 5 is used, the code with getFirstChild / getNodeValue can be replaced by getTextContent.
						Node oTextNode = listResourceNodes.item(k).getFirstChild();
						if (oTextNode != null && oTextNode.getNodeType() == Node.TEXT_NODE)
						{
							if (oTextNode.getNodeValue().equals(sResource))
							{
								m_oResourceViewer.setChecked(aoResourceObjects[j], true);
								checkResourceElement(aoResourceObjects[j], true);				
							}
						}												
					}
				}
			}
			if (iPackageContentProvider.hasChildren(aoPackageObjects[i]))
			{
				checkEnabled(iPackageContentProvider.getChildren(aoPackageObjects[i]), iPackageContentProvider, iResourceContentProvider, listResourceNodes);
			}
		}		
	}
	
	/**
	 * Is called when the selection in the package viewer (tree) changed.
	 *  
	 * @param oEvent Selection changed event.
	 */
	public void selectionChanged(SelectionChangedEvent oEvent)	
	{
		this.m_oSelectedPackageElement = ((IStructuredSelection)oEvent.getSelection()).getFirstElement();
		if (this.m_oSelectedPackageElement != null)
		{
			this.m_oResourceViewer.setInput(this.m_oSelectedPackageElement);
			// Set previous checked elements if necessary
			if (this.m_oSelectedTableElements.get(this.m_oSelectedPackageElement) != null)
			{
				this.m_oResourceViewer.setCheckedElements(((ArrayList)this.m_oSelectedTableElements.get(this.m_oSelectedPackageElement)).toArray());
			}
		}
	}
	
	/**
	 * Creates the widgets for the package / resource selection.
	 * 
	 * @param oParent Parent widget.
	 */
	protected void createPackageSelectionGroup(Composite oParent)
	{
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		Composite container = new Composite(oParent, SWT.NULL);
		container.setLayoutData(gd);
		GridLayout layout = new GridLayout(2, true);
		container.setLayout(layout);

		ITreeContentProvider treeContentProvider = new StandardJavaElementContentProvider()		
		{
			public boolean hasChildren(Object element) 
			{
				// no + should shown in front of packages
				return !(element instanceof IPackageFragment) && super.hasChildren(element);
			}
		};		
		Tree tree = new Tree(container, SWT.CHECK | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);	
		gd.heightHint = 160;
		tree.setLayoutData(gd);
		m_oPackageViewer = new CheckboxTreeViewer(tree);
		m_oPackageViewer.setContentProvider(treeContentProvider);
		m_oPackageViewer.setSorter(new JavaElementSorter());
		m_oPackageViewer.setLabelProvider(new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_BASICS | JavaElementLabelProvider.SHOW_OVERLAY_ICONS | JavaElementLabelProvider.SHOW_SMALL_ICONS));
		m_oPackageViewer.setInput(JavaCore.create(ResourcesPlugin.getWorkspace().getRoot()));//this.m_oParent.getSelectedProject());		
		m_oPackageViewer.addFilter(new PackageViewerFilter());
		m_oPackageViewer.addCheckStateListener(this);
		m_oPackageViewer.addSelectionChangedListener(this);
				
		IStructuredContentProvider tableContentProvider = new StandardJavaElementContentProvider()
		{
			public Object[] getElements(Object oParent)
			{
				Object aoElements[] = super.getElements(oParent);
				int iElementNum = 0;
				for (int i = 0; i < aoElements.length; i++)
				{
					if (aoElements[i] instanceof ICompilationUnit || aoElements[i] instanceof IFile)
						iElementNum++;
				}
				Object aoFilteredElements[] = new Object[iElementNum];
				iElementNum = 0;
				for (int i = 0; i < aoElements.length; i++)
				{
					if (aoElements[i] instanceof ICompilationUnit || aoElements[i] instanceof IFile)
					{
						aoFilteredElements[iElementNum] = aoElements[i];
						iElementNum++;
					}
				}
				
				return aoFilteredElements;
			}
		};
		Table table = new Table(container, SWT.CHECK | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);	
		gd.heightHint = 160;
		table.setLayoutData(gd);
		m_oResourceViewer = new CheckboxTableViewer(table);		
		m_oResourceViewer.setContentProvider(tableContentProvider);//new StandardJavaElementContentProvider());
		m_oResourceViewer.setLabelProvider(new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_BASICS | JavaElementLabelProvider.SHOW_OVERLAY_ICONS | JavaElementLabelProvider.SHOW_SMALL_ICONS));
		m_oResourceViewer.addCheckStateListener(this);
	}
	
	/**
	 * Creates the widgets for the main class selection.
	 * 
	 * @param oParent PArent widget.
	 */
	protected void createMainClassSelectionGroup(Composite oParent)
	{
		Composite oContainer = makeGridContainer(oParent, 3);
		
		m_oMainClassText = addLabelAndText(oContainer, "Main class:", 1, true);
		m_oMainClassBrowseButton = new Button(oContainer, SWT.PUSH);
		m_oMainClassBrowseButton.setText("Browse...");
		m_oMainClassBrowseButton.setEnabled(false);
		m_oMainClassBrowseButton.addSelectionListener(new SelectionAdapter() 
		{
			public void widgetSelected(SelectionEvent e) 
			{
				handleMainClassSelection();
			}
		});
	}

	/**
	 * Handle methode for the main class selection button.
	 */
	private void handleMainClassSelection()
	{
		try
		{
			ProgressMonitorDialog oProgressMonitorDialog = new ProgressMonitorDialog(getShell());
			SelectionDialog oSelectMainClassDialog = 
			JavaUI.createMainTypeDialog(getShell(), oProgressMonitorDialog, getJavaSearchScope(), 0, false);
			oSelectMainClassDialog.setMessage("Select main class");
			oSelectMainClassDialog.open();
			Object[] oResult = oSelectMainClassDialog.getResult();
			if (oResult != null)
			{
				IType oMainClass = (IType) oResult[0];
				this.m_oJarPackageData.setManifestMainClass(oMainClass);
				m_oMainClassText.setText(oMainClass.getFullyQualifiedName());
				dialogChanged();				
			}
		}
		catch (Throwable e)
		{
			e.printStackTrace();
		}
	}	

	/**
	 * Gets all selected Java elements.
	 * 
	 * @return Selected Java elements.
	 */	
	public IJavaElement[] getSelectedJavaElements()
	{
		int iJavaElementNum = 0;
		ArrayList listSelected = getSelectedElements();
		Iterator it = listSelected.iterator();
		while (it.hasNext())
		{
			if (it.next() instanceof IJavaElement)
			{
				iJavaElementNum++;
			}
		}
		IJavaElement aoJavaElements[] = new IJavaElement[iJavaElementNum];
		iJavaElementNum = 0;
		it = listSelected.iterator();
		while (it.hasNext())
		{
			Object oJavaElement = it.next();
			if (oJavaElement instanceof IJavaElement)
			{
				aoJavaElements[iJavaElementNum] = (IJavaElement) oJavaElement;
				iJavaElementNum++;
			}
		}		
		return aoJavaElements;
	}

	/**
	 * Creates a Java search scope for searching over all selected Java elements.
	 * 
	 * @return Java search scope.
	 */	
	public IJavaSearchScope getJavaSearchScope()
	{
		return SearchEngine.createJavaSearchScope(getSelectedJavaElements());
	}
	
	/**
	 * Gets all selected elements.
	 * 
	 * @return Selected elements.
	 */
	public ArrayList getSelectedElements()
	{
		ArrayList listSelected = new ArrayList();		
		Iterator it = this.m_oSelectedTableElements.values().iterator();
		while (it.hasNext())
		{
			listSelected.addAll((Collection)it.next());
		}
		return listSelected;
	}
		
	/**
	 * Gets the full name of the selected main class for the Jar package.
	 * 
	 * @return main class name
	 */
	public String getMainClass()
	{
		return this.m_oMainClassText.getText();
	}

	/**
	 * Sets the main class for the Jar package.
	 * 
	 * @param sMainClass Full name of the name class.
	 * @throws CoreException
	 */
	public void setMainClass(String sMainClass) throws CoreException
	{
		SearchEngine oSearch = new SearchEngine();
		SearchParticipant aoSearchParticipants[] = {SearchEngine.getDefaultSearchParticipant()};
		oSearch.search(SearchPattern.createPattern("main(String[]) void", IJavaSearchConstants.METHOD, IJavaSearchConstants.DECLARATIONS, SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE),
				aoSearchParticipants, getJavaSearchScope(), new MainClassSearchRequestor(sMainClass), null);
	}

	private class MainClassSearchRequestor extends SearchRequestor
	{
		private String m_sMainClassName;
		
		public MainClassSearchRequestor(String sMainClassName)
		{
			m_sMainClassName = sMainClassName;
		}
		
		public void acceptSearchMatch(SearchMatch oSearchMatch) throws CoreException
		{
			if (oSearchMatch.getElement() instanceof IMethod)
			{
				IMethod oMainMethod = (IMethod) oSearchMatch.getElement();
				if (oMainMethod.isMainMethod())
				{
					IType oMainClass = oMainMethod.getDeclaringType();
					if (oMainClass.getFullyQualifiedName().equals(m_sMainClassName))
					{
						m_oJarPackageData.setManifestMainClass(oMainClass);
						m_oMainClassText.setText(oMainClass.getFullyQualifiedName());
						dialogChanged();
					}
				}
			}
		}
		
	}
	
	/**
	 * Is called when the dialog settings have changed.
	*/
	protected void dialogChanged() 
	{
		if (getSelectedJavaElements().length == 0)
		{
			updateStatus("No Java classes selected!");
			this.m_oMainClassBrowseButton.setEnabled(false);
			return;
		}
		this.m_oMainClassBrowseButton.setEnabled(true);
		JarPackageData oJarPackageData = getJarPackageData();
		if (oJarPackageData.getManifestMainClass() == null || oJarPackageData.isMainClassValid(getContainer()) == false)
		{
			updateStatus("Main class must be specified!");
			return;
		}	
		updateStatus(null);
	}

	/**
	 * Filters a elements array with a set of viewer filters.
	 * 
	 * @param aoFilters Set of viewer filters. If null, the outgoing elements are equal the incoming elements.
	 * @param aoElements Incoming elements to filter.
	 * @return Outgoing, filtered, elements.
	 */
	private ArrayList filter(ViewerFilter[] aoFilters, Object[] aoElements) 
	{
		if (aoFilters != null) 
		{
			ArrayList listFiltered = new ArrayList(aoElements.length);
			for (int i = 0; i < aoElements.length; i++) 
			{
				boolean bAdd = true;
				for (int j = 0; j < aoFilters.length; j++) 
				{
					bAdd = aoFilters[j].select(null, null, aoElements[i]);
					if (!bAdd)
						break;
				}
				if (bAdd)
					listFiltered.add(aoElements[i]);
			}
			return listFiltered;
		}
		else
		{
			// No filter, return all objects.
			ArrayList listElements = new ArrayList(aoElements.length);
			for (int i = 0; i < aoElements.length; i++)
			{
				listElements.add(aoElements[i]);
			}	
			return listElements;
		}
	}

	/**
	 * Gets the children of a package element.
	 * 
	 * @param oElement Package element which childrens should returned.
	 * @return Childrens.
	 */
	private ArrayList getPackageChildren(Object oPackageElement) 
	{
		return filter(this.m_oPackageViewer.getFilters(), ((ITreeContentProvider)this.m_oPackageViewer.getContentProvider()).getChildren(oPackageElement));
	}

	/**
	 * Gets the resources of a package element.
	 * 
	 * @param oElement Package element which resources should returned.
	 * @return Resources.
	 */
	private ArrayList getResourceElements(Object oPackageElement) 
	{
		return  filter(this.m_oResourceViewer.getFilters(), ((IStructuredContentProvider)this.m_oResourceViewer.getContentProvider()).getElements(oPackageElement));
	}
	
	/**
	 * Get the jar package data to create a jar package as described with this wizard page.
	 * 
	 * @return Jar package data.
	 */
	public JarPackageData getJarPackageData()
	{
		m_oJarPackageData.setElements(getSelectedElements().toArray());			
		return m_oJarPackageData;
	}
	
	/**
	 * @see ExtWizardPage#writeToProjectResources() 
	 */
	protected void writeToProjectResources()
	{
		writeSettingValuesToXML(this);
		
		Node oRootNode = m_oParent.getDocumentIPKSettings().getDocumentElement();
		if (oRootNode == null)
		{
			oRootNode = m_oParent.getDocumentIPKSettings().createElement("IPKSettings");
			m_oParent.getDocumentIPKSettings().appendChild(oRootNode);
		}	
		Node oChildNode = m_oParent.getDocumentIPKSettings().createElement("JarResources");
		oRootNode.appendChild(oChildNode);
		Iterator it = getSelectedElements().iterator();
		while (it.hasNext())
		{
			IResource iResource = null;
			Object oObject = it.next();
			if (oObject instanceof IResource)
			{
				iResource = (IResource) oObject;
			}
			else if (oObject instanceof IJavaElement)
			{
				IJavaElement iJavaElement = (IJavaElement) oObject;
				iResource = iJavaElement.getResource();
			}
			if (iResource != null)
			{
				setXMLSettingValue(oChildNode, "Resource", iResource.getFullPath().toPortableString());
			}
		}
	}
}

