/*
 * IPK export plugin for Eclipse. 
 * Copyright (C) 2005  Bernd Lachner, InSo GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * You can reach the authr via mail:
 * 
 * InSo GmbH, Individuelle Software-Erstellung
 * Hannongstrae 5
 * 67227 Frankenthal, Germany
 * 
 * or electronic mail:
 * 
 * info@inso-gmbh.de
 */

package de.inso_gmbh.swt.widgets;

import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;

/**
 * @author Bernd Lachner, InSo GmbH
 *
 * Widget for viewing an image.
 */
public class ImageViewer extends Canvas implements PaintListener
{
	protected Image m_oImage = null;
	
	/**
	 * Constructor.
	 * 
	 * @param oParent The widgets parent composite.
	 * @param iStyle The style of the widget. See styles of Canvas class.
	 */
	public ImageViewer(Composite oParent, int iStyle)
	{
		super(oParent, iStyle);
		addPaintListener(this);
	}
	
	/**
	 * Loads an image and shows it if possible.
	 * 
	 * @param sImageName Filename of the image to load.
	 */
	public void loadImage(String sImageName)
	{
		try
		{
			ImageLoader oImageLoader = new ImageLoader();
			this.m_oImage = new Image(this.getDisplay(), oImageLoader.load(sImageName)[0]);
		}
		catch (Exception oException)
		{
			this.m_oImage = null;
		}
		finally
		{
			this.redraw();
		}
	}

	/**
	 * Gets the image actually loaded or null if none is loaded.
	 * 
	 * @return Acual viewed image.
	 */	
	public Image getImage()
	{
		return this.m_oImage;
	}
	
	/**
	 * Clears the viewer.
	 *
	 */
	public void clearImage()
	{
		this.m_oImage = null;
		this.redraw();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.PaintListener#paintControl(org.eclipse.swt.events.PaintEvent)
	 */
	public void paintControl(PaintEvent oPaintEvent) 
	{
		if (this.m_oImage != null)
		{
			oPaintEvent.gc.drawImage(this.m_oImage, 0, 0);
		}
	}
}
